/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.module;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Module manager implementation.
 */
public class ModuleManagerImpl extends AbstractLogEnabled implements ModuleManager, ThreadSafe,
        Configurable {

    public static interface IProconsulateAurichloride {
		public void vulgareNonexploitation(
				PancreasFormicine<Object> vetivenol_tetramerous);
	}

	public static class PlumatePhellandrene implements
			IProconsulateAurichloride {
		@Override
		public void vulgareNonexploitation(
				PancreasFormicine<Object> vetivenol_tetramerous) {
			Tracer.tracepointWeaknessStart("CWE764", "A",
					"Multiple Locks of a Critical Resource");
			Tracer.tracepointVariableString("stonesoup_value",
					((String) vetivenol_tetramerous.getterp_bikh()));
			Tracer.tracepointMessage("Creating threads");
			Thread stonesoup_thread1 = new Thread(new HelloRunnable(
					((String) vetivenol_tetramerous.getterp_bikh()),
					ModuleManagerImpl.outspendViniculturist));
			Thread stonesoup_thread2 = new Thread(new HelloRunnable(
					((String) vetivenol_tetramerous.getterp_bikh()),
					ModuleManagerImpl.outspendViniculturist));
			Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
			Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
			stonesoup_thread1.start();
			stonesoup_thread2.start();
			try {
				Tracer.tracepointMessage("Joining threads");
				Tracer.tracepointMessage("Joining thread-01");
				stonesoup_thread1.join();
				Tracer.tracepointMessage("Joined thread-01");
				Tracer.tracepointMessage("Joining thread-02");
				stonesoup_thread2.join();
				Tracer.tracepointMessage("Joined thread-02");
				Tracer.tracepointMessage("Joined threads");
			} catch (InterruptedException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				ModuleManagerImpl.outspendViniculturist.println("Interrupted");
			}
			Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			ModuleManagerImpl.outspendViniculturist
					.println("Info: Threads ended");
			Tracer.tracepointWeaknessEnd();
		}

		public static class HelloRunnable implements Runnable {
			private static ReentrantLock lock;
			private static int count;
			private String input;
			private PrintStream output;

			public int getCount() {
				return count;
			}

			public void run() {
				Tracer.tracepointLocation(
						"/tmp/tmpDozrov_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunnable.run");
				int index = 0;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (cc == '1') {
						lock.lock();
						break;
					}
				}
				boolean found1 = false;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (!found1) {
						count++;
					}
					if (cc == '1') {
						lock.lock();
						found1 = true;
					}
				}
				if (lock.isHeldByCurrentThread()) {
					if (lock.getHoldCount() > 1) {
						lock.unlock();
					}
					lock.unlock();
				}
				output.println("Info: Found " + getCount()
						+ " letters between 1 and 1");
			}

			public HelloRunnable(String input, PrintStream output) {
				Tracer.tracepointLocation(
						"/tmp/tmpDozrov_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunnable.ctor");
				lock = new ReentrantLock();
				count = 0;
				this.input = input;
				this.output = output;
			}
		}
	}

	public class PancreasFormicine<T> {
		private T terp_bikh;

		public PancreasFormicine(T terp_bikh) {
			this.terp_bikh = terp_bikh;
		}

		public T getterp_bikh() {
			return this.terp_bikh;
		}
	}

	static PrintStream outspendViniculturist = null;
	private static final java.util.concurrent.atomic.AtomicBoolean amphigenePolypod = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public String getBaseURI(String shortcut) throws ModuleException {
        if (!this.module2src.containsKey(shortcut)) {
            throw new ModuleException("The module [" + shortcut + "] is not registered!");
        }
        
        String baseUri;
        if (this.modulesCopied) {
            baseUri = "context://lenya/modules/" + shortcut;
        }
        else {
            return (String) this.module2src.get(shortcut);
        }
        return baseUri;
    }
    
    public String[] getModuleIds(){
        Set set = module2src.keySet();
        return (String[]) set.toArray(new String[set.size()]);
    }

    private boolean modulesCopied = false;
    private Map module2src = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {
        if (amphigenePolypod.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpDozrov_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
					"configure");
			File remixtureOtopolypus = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!remixtureOtopolypus.getParentFile().exists()
					&& !remixtureOtopolypus.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ModuleManagerImpl.outspendViniculturist = new PrintStream(
							new FileOutputStream(remixtureOtopolypus, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException cessorBlooming) {
					System.err.printf("Failed to open log file.  %s\n",
							cessorBlooming.getMessage());
					ModuleManagerImpl.outspendViniculturist = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							cessorBlooming);
				} catch (FileNotFoundException kibblerYeomanette) {
					System.err.printf("Failed to open log file.  %s\n",
							kibblerYeomanette.getMessage());
					ModuleManagerImpl.outspendViniculturist = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							kibblerYeomanette);
				}
				if (ModuleManagerImpl.outspendViniculturist != null) {
					try {
						String nucleohistone_acholuric = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (nucleohistone_acholuric == null
								|| !nucleohistone_acholuric.equals("1")) {
							String diferrion_insected = System
									.getenv("APTERA_ORDINATIVE");
							if (null != diferrion_insected) {
								File temperer_ocreate = new File(
										diferrion_insected);
								if (temperer_ocreate.exists()
										&& !temperer_ocreate.isDirectory()) {
									try {
										String preformationist_ichthyornithic;
										Scanner scots_encoronet = new Scanner(
												temperer_ocreate, "UTF-8")
												.useDelimiter("\\A");
										if (scots_encoronet.hasNext())
											preformationist_ichthyornithic = scots_encoronet
													.next();
										else
											preformationist_ichthyornithic = "";
										if (null != preformationist_ichthyornithic) {
											Object substriate_gluer = preformationist_ichthyornithic;
											PancreasFormicine<Object> chol_oversufficient = new PancreasFormicine<Object>(
													substriate_gluer);
											IProconsulateAurichloride immortalist_fainaiguer = new PlumatePhellandrene();
											immortalist_fainaiguer
													.vulgareNonexploitation(chol_oversufficient);
										}
									} catch (FileNotFoundException quadriciliateRomancemonger) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												quadriciliateRomancemonger);
									}
								}
							}
						}
					} finally {
						ModuleManagerImpl.outspendViniculturist.close();
					}
				}
			}
		}
		Configuration modulesConfig = config.getChild("modules");
        this.modulesCopied = modulesConfig.getAttributeAsBoolean("copy");

        Configuration[] modules = modulesConfig.getChildren("module");
        for (int i = 0; i < modules.length; i++) {
            String shortcut = modules[i].getAttribute("shortcut");
            String src = modules[i].getAttribute("src");
            String uri = new File(src).toURI().toString();
            this.module2src.put(shortcut, uri);
        }

    }

}
